import Axios, { AxiosInstance, AxiosRequestConfig } from 'axios'
import { ElMessage, ElLoading, ILoadingInstance } from 'element-plus'
import Cookies from 'js-cookie'
// import router from '@/router/index'
import { getToken, removeToken } from '@/utils/auth'

interface RequestConfig extends AxiosRequestConfig {
  showLoading?: boolean // 是否显示全屏加载遮罩
  loadingMsg?: string // 加载提示语
  errorMsg?: string // 请求错误默认提示语
  [propName: string]: any
}
export const apiUrl = '' // 'http://192.168.118.135:18000' http://192.168.1.53:18000
const baseURL = '' // 'https://api.github.com'
// const baseURL = 'https://api.github.com'
// 同时发送多次请求显示loading数量
let needLoadingRequestCount: number = 0
// loading对象
let loading: ILoadingInstance
// 存储每个请求的标识和对应的取消函数
const pendingAjax = new Map()
// 根据config返回请求标志
const duplicatedKeyFn = (config: RequestConfig) =>
  `${config.method}${config.url}${JSON.stringify(config.params)}${JSON.stringify(
    config.data
  )}`

/**
 * 将请求添加到pendingAjax
 * @param config
 */
function addPendingAjax(config: RequestConfig) {
  const duplicatedKey = JSON.stringify({
    duplicatedKey: duplicatedKeyFn(config),
    type: 'DUPLICATED_REQUEST'
  })
  config.cancelToken =
    config.cancelToken ||
    new Axios.CancelToken((cancel) => {
      if (duplicatedKey && !pendingAjax.has(duplicatedKey)) {
        // 如果pendingAjax中不存在当前请求，添加进去
        pendingAjax.set(duplicatedKey, cancel)
      }
    })
}
/**
 * 从pendingAjax中删除请求
 * @param config
 */
function removePendingAjax(config: RequestConfig) {
  const duplicatedKey = JSON.stringify({
    duplicatedKey: duplicatedKeyFn(config),
    type: 'DUPLICATED_REQUEST'
  })
  if (duplicatedKey && pendingAjax.has(duplicatedKey)) {
    // 如果pendingAjax中存在当前请求, 取消当前请求并将其删除
    const cancel = pendingAjax.get(duplicatedKey)
    cancel(duplicatedKey)
    pendingAjax.delete(duplicatedKey)
  }
  addPendingAjax(config)
}
/**
 * 显示全屏加载中提示
 * @param loadingMsg 加载显示文本
 */
function tryShowFullScreenLoading(loadingMsg?: string) {
  if (needLoadingRequestCount === 0) {
    loading = ElLoading.service({
      lock: true,
      text: loadingMsg || '加载中',
      spinner: 'el-icon-loading',
      background: 'rgba(0, 0, 0, 0)',
      fullscreen: true,
      customClass: 'loading-style'
    })
  }
  needLoadingRequestCount += 1
}

/**
 * 请求结束后隐藏loading
 * @returns void
 */
function tryHideFullScreenLoading() {
  if (needLoadingRequestCount <= 0) return
  needLoadingRequestCount -= 1
  if (needLoadingRequestCount === 0 && loading) {
    loading.close()
  }
}

/**
 * 创建请求实例
 * @returns axios实例
 */
function createService() {
  const axios: AxiosInstance = Axios.create()
  // 前置拦截器（发起请求之前的拦截）
  axios.interceptors.request.use(
    (config) => {
      /**
       * 根据你的项目实际情况来对 config 做处理
       * 这里对 config 不做任何处理，直接返回
       */
      removePendingAjax(config) // have bug，时序问题
      return config
    },
    (error) => {
      // console.log('request-error:', error)
      return Promise.reject(error)
    }
  )

  // 后置拦截器（获取到响应时的拦截）
  axios.interceptors.response.use(
    (response) => {
      removePendingAjax(response.config)
      const { code, message } = response.data
      if (code === 401 || code === 302) {
        ElMessage.closeAll()
        // 401则跳转到登录页
        ElMessage({
          message: message || '未授权或登录超时，请重新登录',
          type: 'error',
          onClose() {
            removeToken()
            // router.push({ path: '/login' })
            window.location.href = '/#/login'
          }
        })
      }
      return response
    },
    (error) => {
      removePendingAjax(error.config || {})
      // console.log('response-error:', error)
      // 类型是否为重复请求
      // 取消请求会报错，但不应该返回给用户
      let isDuplicatedType
      try {
        const errorType = (JSON.parse(error.message) || {}).type
        isDuplicatedType = errorType === 'DUPLICATED_REQUEST'
      } catch {
        isDuplicatedType = false
      }
      // console.log('isDuplicatedType:', isDuplicatedType)
      if (isDuplicatedType) return

      ElMessage.closeAll()
      // 响应拦截处理在此处处理
      if (error.response && error.response.status) {
        const { status } = error.response
        let msg = ''
        switch (status) {
          case 400:
            msg = '请求错误'
            break
          case 401:
            msg = '未授权或登录超时，请重新登录'
            break
          case 403:
            msg = '拒绝访问'
            break
          case 404:
            msg = '请求地址出错'
            break
          case 408:
            msg = '请求超时'
            break
          case 500:
            msg = '服务器内部错误'
            break
          case 501:
            msg = '服务未实现'
            break
          case 502:
            msg = '网关错误'
            break
          case 503:
            msg = '服务不可用'
            break
          case 504:
            msg = '网关超时'
            break
          case 505:
            msg = 'HTTP版本不受支持'
            break
          default:
            msg = '连接出错'
            break
        }
        if (status === 401) {
          // 401则跳转到登录页

          ElMessage.closeAll()
          ElMessage({
            message: '未授权或登录超时，请重新登录',
            type: 'error',
            onClose() {
              removeToken()
              // router.push({ path: '/login' })
              window.location.href = '/#/login'
            }
          })
        } else {
          ElMessage.error(`${msg}`)
        }
      } else {
        ElMessage.error('连接出错')
      }
      // return Promise.reject(error)
    }
  )
  return axios
}

// 用于网络请求的实例和请求方法
export const service = createService()

// 创建请求方法
function createRequestFunction() {
  return (config: RequestConfig) => {
    const configDefault = {
      headers: {
        Authorization: `${Cookies.get('Token') || ''}`,
        'Content-Type': config.contentType || 'application/json' // formData形式：application/x-www-form-urlencoded;charset=UTF-8
      },
      baseURL,
      timeout: 120000, // 请求超时 20s
      data: {},
      method: 'get',
      needHandleRes: true, // 是否需要处理response，如证书下载直接返回response
      // showLoading: false,
      // loadingMsg: '加载中',
      needErrorMsg: true // 是否需要显示错误信息
    }
    const mergeConfig = Object.assign(configDefault, config)

    if (config.showLoading) {
      tryShowFullScreenLoading(config.loadingMsg)
    }
    return service(mergeConfig)
      .then(
        (response) => {
          const address = `${window.location.protocol}//${window.location.host}/login`

          // 处理302重定向问题
          if (response && response.request.responseURL === address) {
            window.location.href = `${window.location.protocol}//${window.location.host}/#/login`
            ElMessage.closeAll()
            ElMessage({
              message: '未授权或登录超时，请重新登录',
              type: 'error'
            })
          }
          if (config.showLoading) {
            tryHideFullScreenLoading()
          }
          // 此处需要统一处理返回状态、数据、错误提示等
          // return response
          // 验证码校验时已经有Token时表示已登录成功，直接返回
          if (mergeConfig.url === '/api/common/checkCaptcha' && getToken()) {
            return null
          }
          if (mergeConfig.needHandleRes) {
            const { code, message } = response.data
            if (code === 200) {
              return response
            }
            if (code === 401 || code === 302) {
              ElMessage.closeAll()
              // 401则跳转到登录页
              ElMessage({
                message: message || '未授权或登录超时，请重新登录',
                type: 'error',
                onClose() {
                  removeToken()
                  // router.push({ path: '/login' })
                  window.location.href = '/#/login'
                }
              })
            } else {
              if ((mergeConfig.errorMsg || message) && mergeConfig.needErrorMsg) {
                ElMessage.closeAll()
                ElMessage.error(mergeConfig.errorMsg || message)
              }
              return Promise.reject()
            }
          }
          return response
        },
        () => {}
      )
      .catch((error) => {
        if (mergeConfig.showLoading) {
          tryHideFullScreenLoading()
        }
        return Promise.reject(error)
      })
  }
}

// 用于网络请求的实例和请求方法
export const request = createRequestFunction()
