import * as lodash from 'lodash'
import { ElMessageBox, ElMessage } from 'element-plus'

export default class Decorator {
  /**
   * 函数节流装饰器
   * @param {number} wait 节流的毫秒
   * @param {Object} options 节流选项对象
   * [options.leading=true] (boolean): 指定调用在节流开始前。
   * [options.trailing=true] (boolean): 指定调用在节流结束后。
   */
  public static throttle(wait: number | undefined, options = {}) {
    return (
      target: any,
      name: any,
      descriptor: { value: lodash.DebouncedFunc<any> }
    ) => {
      // eslint-disable-next-line no-param-reassign
      descriptor.value = lodash.throttle(descriptor.value, wait, options)
    }
  }

  /**
   * 函数防抖装饰器
   * @param {number} wait 需要延迟的毫秒数。
   * @param {Object} options 选项对象
   * [options.leading=false] (boolean): 指定在延迟开始前调用。
   * [options.maxWait] (number): 设置 func 允许被延迟的最大值。
   * [options.trailing=true] (boolean): 指定在延迟结束后调用。
   */
  public static debounce(wait: number | undefined, options = {}) {
    return (
      target: any,
      name: any,
      descriptor: { value: lodash.DebouncedFunc<any> }
    ) => {
      // eslint-disable-next-line no-param-reassign
      descriptor.value = lodash.debounce(descriptor.value, wait, options)
    }
  }

  /**
   * 确认提示框装饰器
   * @param {*} message 提示信息
   * @param {*} title 标题
   * @param {*} cancelFn 取消回调函数
   */
  public static confirm(message = '确定要删除数据，此操作不可回退。', title = '提示') {
    return (_target: any, _name: any, descriptor: any) => {
      const originFn = descriptor.value
      // eslint-disable-next-line no-param-reassign
      descriptor.value = async (...rest: any) => {
        try {
          await ElMessageBox.confirm(message, title, {
            confirmButtonText: '确定',
            cancelButtonText: '取消',
            type: 'warning'
          })
          // .then(() => {
          //   ElMessage.success({
          //     message: '删除成功!'
          //   })
          // })
          // .catch(() => {
          //   ElMessage.info({
          //     message: '已取消删除'
          //   })
          // })
          originFn.apply(this, rest)
        } catch (error) {
          ElMessage.info({
            message: '已取消删除'
          })
        }
      }
    }
  }
}
